#
#       HBS REDUCTION SUB ROUTINE
#
#        Get Photometoric Parameters by fitting : hbsphfit2.cl
#
#       1998/07/31      K.Kawabata  Ver.2.00
#       2005/04/07      M.Isogai    Ver.2.10 for iraf v2.12
#

procedure hbsphfit2( inlist, convf_fn, extn_fn )

file inlist { prompt = "Input table" }
file convf_fn { prompt = "Output image of conversion factor" }
file extn_fn { prompt = "Output image of extinction" }

begin

  file infn, outfn1, outfn2

  real am1, am2, am3, am4
  real cof1[7], cof2[7], cof3[7], cof4[7]
  real beta1[7], beta2[7]
  int nord = 7
  file fn_tpl = "hbs.tpl.i2"
  real e0 = 3.67e-20 # erg/cm^2/sec/Hz
  real meanmag = 17.0
  real e1

  string buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7, buf8
  string tmp_fn1, tmp_fn2
  real cdelt, crval, crpix
  int npix

  int c, i, j
  real lmd, a1, a2

  int flag_ver       # 0:no display 1:display
  int n_version      # IRAF version (210 or 211)

#--- initial settings

  tmp_fn1 = mktemp( "hbs.tmp.fn." )
  tmp_fn2 = mktemp( "hbs.tmp.fn." )

  infn = inlist
  outfn1 = convf_fn
  outfn2 = extn_fn

  print( "" )
  print( "------------------------------" )
  print( " Photmetric Parameter Fitting"
  print( "------------------------------" )
  print( "" )

#--- Verbose Mode

  if( envget( "_fg_ver" ) == "1" )
  {
    flag_ver = 1
  }
  else
  {
    if( envget( "_fg_ver" ) == "2" )
    {
      flag_ver = 0
    }
    else
    {
      printf( "'hbsinit' process is incomplete.(_fg_ver) Retry.\n" )
      bye
    }
  }

#----- IRAF version check

  buf1 = envget( "version" )
  c = stridx( "V", buf1 )
  buf2 = substr( buf1, c, ( c + 4 ) )

  n_version = 0

  if( buf2 == "V2.11" || buf2 == "V2.12" )
  {
    n_version = 211
  }
  else
  {
    if( buf2 == "V2.10" )
    {
      n_version = 210
    }
  }

  if( n_version == 0 )
  {
    printf( "IRAF version check failed.\n" )
    printf( "Please input like next line, and restart 'hbsred'\n"
    printf( "cl> set version V2.10  (V2.10 and V2.11 are valid)\n"
    bye
  }

#--- check template image

  if( access( fn_tpl//".imh" ) == 0 )
  {
    printf( "'%s' not found. Please do 'hbsphfit1' again.\n", fn_tpl )
    bye
  }
  imgets( fn_tpl, "CRPIX1" )
  crpix = real( imgets.value )
  imgets( fn_tpl, "CRVAL1" )
  crval = real( imgets.value )
  imgets( fn_tpl, "CD1_1" )
  cdelt = real( imgets.value )
  imgets( fn_tpl, "i_naxis1" )
  npix = int( imgets.value )

  if( crval < 1000 )
  {
    printf( "Maybe something is wrong in wavelength calibration in %s\n",\
            fn_tpl )
    printf( "Change order in the input-list of 'hbsphfit1', and do again.\n" )
    bye
  }

#----- Scanning the list file

  buf = ""

  if( access( infn ) == 0 )
  {
    print( "Cannot find "//infn//"." )
    bye
  }

  if( access( outfn1 ) == 1 || access( outfn1//".imh" ) == 1 )
  {
    printf( "'%s' already exists. Replace? [y/n] >>", outfn1 )
    c = scan( buf )
    while( buf != "y" && buf != "n" )
      c = scan( buf )
    if( buf == "n" )
      bye
    imdelete( outfn1, ver- )
  }
  if( access( outfn2 ) == 1 || access( outfn2//".imh" ) == 1 )
  {
    printf( "'%s' already exists. Replace? [y/n] >>", outfn2 )
    c = scan( buf )
    while( buf != "y" && buf != "n" )
      c = scan( buf )
    if( buf == "n" )
      bye
    imdelete( outfn2, ver- )
  }

  if( access( tmp_fn1 ) == 1 )
    delete( tmp_fn1, ver- )
  if( access( tmp_fn2 ) == 1 )
    delete( tmp_fn2, ver- )


#--- read input list

  list = infn
  if( fscan( list, buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7 ) != 8 )
    bye
  am1 = real( buf )
  cof1[1] = real( buf1 )
  cof1[2] = real( buf2 )
  cof1[3] = real( buf3 )
  cof1[4] = real( buf4 )
  cof1[5] = real( buf5 )
  cof1[6] = real( buf6 )
  cof1[7] = real( buf7 )

  if( fscan( list, buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7 ) != 8 )
    bye
  am2 = real( buf )
  cof2[1] = real( buf1 )
  cof2[2] = real( buf2 )
  cof2[3] = real( buf3 )
  cof2[4] = real( buf4 )
  cof2[5] = real( buf5 )
  cof2[6] = real( buf6 )
  cof2[7] = real( buf7 )

  if( fscan( list, buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7 ) != 8 )
    bye
  am3 = real( buf )
  cof3[1] = real( buf1 )
  cof3[2] = real( buf2 )
  cof3[3] = real( buf3 )
  cof3[4] = real( buf4 )
  cof3[5] = real( buf5 )
  cof3[6] = real( buf6 )
  cof3[7] = real( buf7 )

  if( fscan( list, buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7 ) != 8 )
    bye
  am4 = real( buf )
  cof4[1] = real( buf1 )
  cof4[2] = real( buf2 )
  cof4[3] = real( buf3 )
  cof4[4] = real( buf4 )
  cof4[5] = real( buf5 )
  cof4[6] = real( buf6 )
  cof4[7] = real( buf7 )

#----- main routine

  for( i=1; i<=nord; i+=1 )
  {
    if( flag_ver == 1 )
    printf( "Coefficient %d \n", i )

#---- make temprary file

    printf( "%7.4f  %10e\n", am1, cof1[i], >> tmp_fn1 )
    printf( "%7.4f  %10e\n", am2, cof2[i], >> tmp_fn1 )
    printf( "%7.4f  %10e\n", am3, cof3[i], >> tmp_fn1 )
    printf( "%7.4f  %10e\n", am4, cof4[i], >> tmp_fn1 )

#---- fitting

    curfit( tmp_fn1, function="legendre", weighting="uniform",\
            order=2, interactive+, axis=1, listdata-, ver-,\
            calctype="double", power+, device="stdgraph", cursor="",\
            > tmp_fn2 )
    delete( tmp_fn1, ver- )
#   type( tmp_fn2 )

    list = tmp_fn2
    if( n_version == 210 )
    {
      while( fscan( list, buf, buf1, buf2, buf3, buf4, buf5,\
                     buf6, buf7, buf8 ) != EOF )
      {
        if( strlen( buf ) != 1 )
          next
        j = int( buf )
        if( j == 1 )
          beta1[i] = real( buf1 )
        if( j == 2 )
          beta2[i] = real( buf1 )
      }
    }
    else
    {
      while( fscan( list, buf8, buf, buf1, buf2, buf3, buf4, buf5,\
                     buf6, buf7, buf8 ) != EOF )
      {
        if( strlen( buf ) != 1 )
          next
        j = int( buf )
        if( j == 1 )
          beta1[i] = real( buf1 )
        if( j == 2 )
          beta2[i] = real( buf1 )
      }
    }
    delete( tmp_fn2, ver- )

    printf( "Coef. %d   beta1=%10e beta2=%10e\n", i, beta1[i], beta2[i] )

  }

#--- Making images

  printf( "Now making '%s' and '%s' images.\n", outfn1, outfn2 )
  imcopy( fn_tpl, outfn1, ver- )
  imcopy( fn_tpl, outfn2, ver- )

  e1 = e0 * 10**( -0.4*meanmag )
  for( j=1; j<=npix; j+=1 )
  {
    lmd = crval + ( j - crpix ) * cdelt
    a1 = beta1[1] + beta1[2]*lmd + beta1[3]*lmd*lmd + beta1[4]*(lmd**3)\
         + beta1[5]*(lmd**4) + beta1[6]*(lmd**5) + beta1[7]*(lmd**6)
    a2 = beta2[1] + beta2[2]*lmd + beta2[3]*lmd*lmd + beta2[4]*(lmd**3)\
         + beta2[5]*(lmd**4) + beta2[6]*(lmd**5) + beta2[7]*(lmd**6)

    imreplace( outfn1//"["//j//":"//j//"]", ( exp( a1 ) * e1 ) )
    imreplace( outfn2//"["//j//":"//j//"]", a2 )
  }

end
